@@ -1,4 +1,5 @@
 use std::convert::TryFrom;
+use std::num::TryFromIntError;
 
 #[cfg(test)]
 mod tests;
@@ -11,12 +12,21 @@ pub enum GasLimit {
 #[derive(Debug)]
 pub enum StackMachineError {
     UnkownError,
+    NumericOverflow,
     NumberStackUnderflow,
     LoopStackUnderflow,
     UnhandledTrap,
     RanOutOfGas,
 }
 
+impl From<TryFromIntError> for StackMachineError {
+    fn from(err: TryFromIntError) -> StackMachineError {
+        match err {
+            _ => StackMachineError::NumericOverflow,
+        }
+    }
+}
+
 pub enum TrapHandled {
     Handled,
     NotHandled,
@@ -42,7 +52,7 @@ impl<'a> TrapHandler<'a> {
         C: Fn(i64, &mut StackMachineState) -> Result<TrapHandled, StackMachineError> + 'a,
     {
         TrapHandler {
-            handled_trap: handled_trap,
+            handled_trap,
             to_run: Box::new(f),
         }
     }
@@ -101,8 +111,8 @@ pub struct StackMachineState {
     gas_used: u64,
 }
 
-impl StackMachineState {
-    pub fn new() -> StackMachineState {
+impl Default for StackMachineState {
+    fn default() -> Self {
         StackMachineState {
             number_stack: Vec::new(),
             return_stack: Vec::new(),
@@ -125,22 +135,24 @@ pub struct StackMachine {
     pub trap_handlers: Vec<Box<dyn HandleTrap>>,
 }
 
-impl StackMachine {
-    pub fn new() -> StackMachine {
+impl Default for StackMachine {
+    fn default() -> StackMachine {
         StackMachine {
-            st: StackMachineState::new(),
+            st: StackMachineState::default(),
             trap_handlers: Vec::new(),
         }
     }
+}
 
+impl StackMachine {
     /// JR(*) is relative from the JR(*) instruction,
     /// 0 would jump back onto the JR instruction
     /// -1 Would jump back to the instruction before the JR(*}) instruction
     /// 1 Would jump to the instruction after the JR(*) instruction
     ///
     /// TRAPs always have a numeric code on the number stack to define which TRAP is being called
-    /// 
-    /// CMPLOOP 
+    ///
+    /// CMPLOOP
     /// pushes 1 on the stack if the loop counter is greater than or equal to the max
     /// pushes 0 on the stack if the loop counter is less than the max
     pub fn execute(
@@ -163,13 +175,12 @@ impl StackMachine {
                     pc_reset = true;
                 }
                 Opcode::JR => {
-                    let new_offset = self.st.pc as i128
+                    let new_offset = i64::try_from(self.st.pc)?
                         + self
                             .st
                             .number_stack
                             .pop()
-                            .ok_or(StackMachineError::NumberStackUnderflow)?
-                            as i128;
+                            .ok_or(StackMachineError::NumberStackUnderflow)?;
                     self.st.pc = usize::try_from(new_offset).unwrap();
                     pc_reset = true;
                 }
@@ -208,13 +219,12 @@ impl StackMachine {
                     }
                 }
                 Opcode::JRZ => {
-                    let new_offset = self.st.pc as i128
+                    let new_offset = i64::try_from(self.st.pc)?
                         + self
                             .st
                             .number_stack
                             .pop()
-                            .ok_or(StackMachineError::NumberStackUnderflow)?
-                            as i128;
+                            .ok_or(StackMachineError::NumberStackUnderflow)?;
                     let x = self
                         .st
                         .number_stack
@@ -226,13 +236,12 @@ impl StackMachine {
                     }
                 }
                 Opcode::JRNZ => {
-                    let new_offset = self.st.pc as i128
+                    let new_offset = i64::try_from(self.st.pc)?
                         + self
                             .st
                             .number_stack
                             .pop()
-                            .ok_or(StackMachineError::NumberStackUnderflow)?
-                            as i128;
+                            .ok_or(StackMachineError::NumberStackUnderflow)?;
                     let x = self
                         .st
                         .number_stack
@@ -316,7 +325,10 @@ impl StackMachine {
                         .number_stack
                         .pop()
                         .ok_or(StackMachineError::NumberStackUnderflow)?;
-                    self.st.number_stack.push(!x);
+                    self.st.number_stack.push(match x {
+                        0 => 1,
+                        _ => 0,
+                    });
                 }
                 Opcode::DUP => {
                     let x = self
@@ -370,7 +382,7 @@ impl StackMachine {
                 }
                 Opcode::INCLP => match self.st.loop_stack.last_mut() {
                     Some((current_index, _max_index)) => {
-                        *current_index = *current_index + 1;
+                        *current_index += 1;
                     }
                     None => {
                         return Err(StackMachineError::LoopStackUnderflow);
@@ -385,7 +397,7 @@ impl StackMachine {
 
                     match self.st.loop_stack.last_mut() {
                         Some((current_index, _max_index)) => {
-                            *current_index = *current_index + increment;
+                            *current_index += increment;
                         }
                         None => {
                             return Err(StackMachineError::LoopStackUnderflow);
@@ -401,6 +413,9 @@ impl StackMachine {
                     self.st.number_stack.push(*current_index);
                 }
                 Opcode::GETLP2 => {
+                    if self.st.loop_stack.len() < 2 {
+                        return Err(StackMachineError::LoopStackUnderflow);
+                    }
                     let (current_index, _max_index) = self
                         .st
                         .loop_stack
@@ -428,7 +443,7 @@ impl StackMachine {
                     }
                 }
             };
-            if pc_reset == false {
+            if !pc_reset {
                 self.st.pc += 1;
             }
 
